SuperStrict

Framework BRL.Basic
Import BRL.Max2D
Import BRL.Timer
'Import BRL.D3D7Max2D
Import BRL.GLMax2D
Import BRL.PNGLoader

Const GWIDTH:Int  = 400
Const GHEIGHT:Int = 400

Graphics GWIDTH, GHEIGHT

Local Physics:TPhysics = TPhysics.Create()
Local SphereElevator:TSphereElevator = InitClock( Physics )

Local BGImage:TImage = LoadImage( "Gfx/BG.png" )

Local Timer:TTimer = CreateTimer( 60 )

While Not ( KeyHit( KEY_ESCAPE ) Or AppTerminate() )
	Cls
	
	Local Counter:Int = MilliSecs()
	
	Physics.Update()
	
	SphereElevator.Render()
	
	DrawImage BGImage, 0, 0
	
	Physics.Render()
	
	If MouseHit( 1 ) Then Physics.AddDynamicBody( TSphere.Create( Vec2( MouseX(), MouseY() ), 10, 0.1 ) )
	
	Flip 0
	WaitTimer Timer
	
	If MilliSecs() - Counter > 100 Then SphereElevator = InitClock( Physics )
Wend
End

Function PointDistance:Float( V1:TVector, V2:TVector )
	Return V1.Copy().Substract( V2 ).Length()
End Function

Function Vec2:TVector( X:Float, Y:Float )
	Return TVector.Vector( X, Y )
End Function

Type TVector
	Field X:Float
	Field Y:Float
	
	Method Add:TVector( B:TVector )
		X :+ B.X
		Y :+ B.Y
		
		Return Self
	End Method
	
	Method Substract:TVector( B:TVector )
		X :- B.X
		Y :- B.Y
		
		Return Self
	End Method
	
	Method Multiply:TVector( Scalar:Float )
		X :* Scalar
		Y :* Scalar
		
		Return Self
	End Method
	
	Method Divide:TVector( Scalar:Float )
		X :/ Scalar
		Y :/ Scalar
		
		Return Self
	End Method
	
	Method DotP:Float( B:TVector )
		Return X*B.X + Y*B.Y
	End Method
	
	Method Length:Float()
		Return Sqr( X*X + Y*Y )
	End Method
	
	Method Normalize:TVector()
		Local Length:Float = Sqr( X*X + Y*Y )
		
		X = X/Length
		Y = Y/Length
		
		Return Self
	End Method
	
	Method Copy:TVector()
		Return Vector( X, Y )
	End Method
	
	Function Vector:TVector( X:Float, Y:Float )
		Local Vector:TVector = New TVector
			Vector.X = X
			Vector.Y = Y
		
		Return Vector
	End Function
End Type

Type TPhysics
	Field _FetchSizeD:Int
	Field _FetchSizeS:Int
	
	Field DynamicBodyCount:Int
	Field StaticBodyCount:Int
	
	Field Iterations:Int = 20
	
	Field DynamicBodies:TDynamicBody[]
	Field StaticBodies:TStaticBody[]
	
	Method AddDynamicBody( DynamicBody:TDynamicBody )
		If DynamicBodyCount >= DynamicBodies.Length Then DynamicBodies :+ New TSphere[ _FetchSizeD ]
		
		DynamicBodies[ DynamicBodyCount ] = DynamicBody
		DynamicBodyCount :+ 1
	End Method
	
	Method AddStaticBody( StaticBody:TStaticBody )
		If StaticBodyCount >= StaticBodies.Length Then StaticBodies :+ New TStaticBody[ _FetchSizeS ]
		
		StaticBodies[ StaticBodyCount ] = StaticBody
		StaticBodyCount :+ 1
	End Method
	
	Method SetFetchSize( FetchSizeD:Int, FetchSizeS:Int )
		_FetchSizeD = Max( FetchSizeD, 1 )
		_FetchSizeS = Max( FetchSizeS, 1 )
	End Method
	
	Method Update()
		For Local DynamicBody:TDynamicBody = EachIn DynamicBodies
			DynamicBody.Update()
		Next
		
		For Local Iteration:Int = 0 To Iterations - 1
			For Local DynamicBody:TDynamicBody = EachIn DynamicBodies
				For Local StaticBody:TStaticBody = EachIn StaticBodies
					If Not BodiesOverlap( DynamicBody, StaticBody ) Then Continue
					
					Local CollInfo:TCollisionInfo = StaticBody.DetectIntersection( DynamicBody )
					
					If CollInfo Then DynamicBody.ApplyImpulse( CollInfo )
				Next
			Next
			
			For Local I:Int = 0 To DynamicBodyCount - 2
				For Local T:Int = I + 1 To DynamicBodyCount - 1
					If Not BodiesOverlap( DynamicBodies[ I ], DynamicBodies[ T ] ) Then Continue
					
					Local CollInfo1:TCollisionInfo = DynamicBodies[ I ].DetectIntersection( DynamicBodies[ T ] )
					
					If CollInfo1 Then
						Local CollInfo2:TCollisionInfo = CollInfo1.Copy()
						CollInfo1.CollisionNormal.Multiply( -1.0 )
						
						Local Mass1:Float = DynamicBodies[ I ].GetMass( CollInfo1.Intersection )
						Local Mass2:Float = DynamicBodies[ T ].GetMass( CollInfo1.Intersection )
						
						CollInfo1.CollisionLength :* Mass2/( Mass1 + Mass2 )
						CollInfo2.CollisionLength :* Mass1/( Mass1 + Mass2 )
						
						DynamicBodies[ I ].ApplyImpulse( CollInfo1 )
						DynamicBodies[ T ].ApplyImpulse( CollInfo2 )
					EndIf
				Next
			Next
		Next
	End Method
	
	Method Render()
		For Local DynamicBody:TDynamicBody = EachIn DynamicBodies
			DynamicBody.Render()
		Next
	End Method
	
	Method Clear()
		DynamicBodies = Null
		StaticBodies  = Null
		
		DynamicBodyCount = 0
		StaticBodyCount  = 0
		
		GCCollect()
	End Method
	
	Function BodiesOverlap:Int( B1:TPhysicsBody, B2:TPhysicsBody )
		Local MinX1:Float, MinY1:Float, MaxX1:Float, MaxY1:Float
		Local MinX2:Float, MinY2:Float, MaxX2:Float, MaxY2:Float
		
		B1.GetBoundingBox( MinX1, MinY1, MaxX1, MaxY1 )
		B2.GetBoundingBox( MinX2, MinY2, MaxX2, MaxY2 )
		
		If MinX1 <= MaxX2 And MinY1 <= MaxY2 And MaxX1 >= MinX2 And MaxY1 >= MinY2 Then Return True
	End Function
	
	Function Create:TPhysics( FetchSizeD:Int = 32, FetchSizeS:Int = 32 )
		Local Physics:TPhysics = New TPhysics
		
		Physics.DynamicBodies = New TDynamicBody[ 1 ]
		Physics.StaticBodies = New TStaticBody[ 1 ]
		Physics.SetFetchSize( FetchSizeD, FetchSizeS )
		
		Return Physics
	End Function
End Type

Type TPhysicsBody Abstract
	Const KIND_SPHERE:Int	= 1
	Const KIND_SEESAW:Int	= 2
	
	Global Gravity:TVector = Vec2( 0, 1 )
	
	Field Visible:Int = True
	
	Method Render() Abstract
	Method GetBoundingBox( MinX:Float Var, MinY:Float Var, MaxX:Float Var, MaxY:Float Var ) Abstract
End Type

Type TStaticBody Extends TPhysicsBody Abstract
	Method DetectIntersection:TCollisionInfo( DynamicBody:TDynamicBody ) Abstract
End Type

Type TDynamicBody Extends TStaticBody Abstract
	Field Kind:Int
	
	Method Update() Abstract
	Method ApplyImpulse( CollInfo:TCollisionInfo ) Abstract
	Method GetMass:Float( Position:TVector ) Abstract
End Type

Type TCollisionInfo
	Field Intersection:TVector
	Field CollisionNormal:TVector
	Field CollisionLength:Float
	
	Method Copy:TCollisionInfo()
		Return Create( Intersection, CollisionNormal.Copy(), CollisionLength )
	End Method
	
	Function Create:TCollisionInfo( pIntersection:TVector, pCollisionNormal:TVector, pCollisionLength:Float )
		Local CollInfo:TCollisionInfo = New TCollisionInfo
		
		CollInfo.Intersection    = pIntersection
		CollInfo.CollisionNormal = pCollisionNormal
		CollInfo.CollisionLength = pCollisionLength
		
		Return CollInfo
	End Function
End Type

Type TSphereElevator Extends TDynamicBody
	Global WheelImage:TImage
	Global ElevatorImg:TImage
	
	Field ElevatorCar1:TLine
	Field ElevatorCar2:TLine
	
	Field ECDistance1:Float
	Field ECDistance2:Float
	Field ECDistance3:Float
	Field ECDeltaAngle:Float
	
	Field ElevatorRotation:Float
	
	Field S1:TSphere
	Field S2:TSphere
	
	Field Position:TVector
	Field Size:TVector
	
	Field LastSecond:Int
	Field LastSystemTime:Int
	
	Method Update()
		Local Seconds:Int = CurrentTime()[ 6 .. ].ToInt(), Ratio:Float
		
		If Seconds <> LastSecond Then
			Local DeltaTime:Int = MilliSecs() - LastSystemTime
			
			Ratio = ( Seconds + ( DeltaTime - 1000 )*0.001 )/60.0
			
			LastSecond = Seconds
			LastSystemTime :+ 1000
		Else
			Local DeltaTime:Int = MilliSecs() - LastSystemTime
			
			Ratio = ( Seconds + DeltaTime*0.001 )/60.0
		EndIf
		
		SetRatio( Ratio )
	End Method
	
	Method SetRatio( Ratio:Float )
		Local CircumCircle:Float = Size.X*Pi
		
		Local CircleRatio:Float = 0.5*CircumCircle/( CircumCircle + Size.Y*2 - Size.X*2 )
		Local RopeRatio:Float = ( Size.Y - Size.X )/( CircumCircle + Size.Y*2 - Size.X*2 )
		
		If Ratio <= CircleRatio/2.0 Or Ratio >= RopeRatio*2.0 + CircleRatio*1.5 Then
			If Ratio >= RopeRatio*2.0 + CircleRatio*1.5 Then Ratio = Ratio - RopeRatio*2.0
			Local Angle:Float = 90 - Ratio/( 2.0*CircleRatio )*360
			
			SetCircleRatio( Angle, S1.Position )
			
			ElevatorRotation = -Angle
		ElseIf Ratio >= CircleRatio/2.0 + RopeRatio And Ratio <= CircleRatio*1.5 + RopeRatio Then
			Local Angle:Float = -( Ratio - CircleRatio/2.0 - RopeRatio )/( 2.0*CircleRatio )*360
			
			SetCircleRatio( Angle, S2.Position )
			
			ElevatorRotation = -Angle
		ElseIf Ratio <= RopeRatio + CircleRatio/2.0 Then
			Ratio = ( Ratio - CircleRatio/2.0 )/RopeRatio
			
			SetRightRopeRatio( Ratio )
			
			ElevatorRotation = 0
		Else
			Ratio = ( Ratio - CircleRatio*1.5 - RopeRatio )/RopeRatio
			
			SetLeftRopeRatio( Ratio )
			
			ElevatorRotation = 180
		EndIf
	End Method
	
	Method SetLeftRopeRatio( Ratio:Float )
		Local Y:Float = Position.Y - Size.X/2.0 + Size.Y - ( Size.Y - Size.X )*Ratio
		
		Local Terminal1:TVector = Vec2( Position.X, Y ) 
		Local Terminal2:TVector = Vec2( Position.X - 10, Y + 10 )
		Local Terminal3:TVector = Vec2( Position.X - 20, Y )
		
		ElevatorCar1.SetTerminals( Terminal1, Terminal2 )
		ElevatorCar2.SetTerminals( Terminal2, Terminal3 )
	End Method
	
	Method SetRightRopeRatio( Ratio:Float )
		Local Y:Float = Position.Y + Size.X/2.0 + ( Size.Y - Size.X )*Ratio
		
		Local Terminal1:TVector = Vec2( Position.X + Size.X, Y ) 
		Local Terminal2:TVector = Vec2( Position.X + Size.X + 10, Y - 10 )
		Local Terminal3:TVector = Vec2( Position.X + Size.X + 20, Y )
		
		ElevatorCar1.SetTerminals( Terminal1, Terminal2 )
		ElevatorCar2.SetTerminals( Terminal2, Terminal3 )
	End Method
	
	Method SetCircleRatio( Angle:Float, Center:TVector )
		Local Terminal1:TVector = Center.Copy().Add( Vec2( ECDistance1*Cos( Angle ), -ECDistance1*Sin( Angle ) ) )
		Local Terminal2:TVector = Center.Copy().Add( Vec2( ECDistance2*Cos( Angle + ECDeltaAngle ), -ECDistance2*Sin( Angle + ECDeltaAngle ) ) )
		Local Terminal3:TVector = Center.Copy().Add( Vec2( ECDistance3*Cos( Angle ), -ECDistance3*Sin( Angle ) ) )
		
		ElevatorCar1.SetTerminals( Terminal1, Terminal2 )
		ElevatorCar2.SetTerminals( Terminal2, Terminal3 )
	End Method
	
	Method ApplyImpulse( CollInfo:TCollisionInfo )
	End Method
	
	Method GetMass:Float( Point:TVector )
		Return 0
	End Method
	
	Method DetectIntersection:TCollisionInfo( DynamicBody:TDynamicBody )
		Return Null
	End Method
	
	Method GetBoundingBox( MinX:Float Var, MinY:Float Var, MaxX:Float Var, MaxY:Float Var )
		MinX = -100; MinY = -100
		MaxX = -100; MaxY = -100
	End Method
	
	Method Render()
		If Visible Then
			Local Circumference:Float = Size.X*Pi + Size.Y*2 - Size.X*2
			Local TurnPerSecond:Float = 360.0*Circumference/( Size.X*Pi*60.0 )
			
			Local Rotation:Float = ( MilliSecs() Mod Int( 360.0*1000.0/TurnPerSecond ) )*0.001*TurnPerSecond
			
			SetRotation( Rotation )
			SetImageHandle( WheelImage, Size.X/2.0, Size.X/2.0 )
			DrawImage WheelImage, Position.X + Size.X/2.0, Position.Y + Size.X/2.0
			
			SetRotation( Rotation + 65 )
			DrawImage WheelImage, Position.X + Size.X/2.0, Position.Y + Size.Y - Size.X/2.0
			
			SetRotation( 0 )
		Else
			SetImageHandle( ElevatorImg, 0, 13 )
			SetRotation( ElevatorRotation )
			
			DrawImage ElevatorImg, ElevatorCar1._X1.X, ElevatorCar1._X1.Y
			
			SetRotation( 0 )
		EndIf
		
		Visible = Not Visible
	End Method
	
	Function Create:TSphereElevator( Physics:TPhysics, Position:TVector, Width:Int, Height:Int )
		If Not WheelImage  Then WheelImage  = LoadImage( "Gfx/Wheel.png" )
		If Not ElevatorImg Then ElevatorImg = LoadImage( "Gfx/ElevatorCar.png" )
		
		Local Elevator:TSphereElevator = New TSphereElevator
		
		Local CabinPos:TVector = Position.Copy().Add( Vec2( Width, Height/2 ) )
		
		Elevator.ElevatorCar1	= TLine.Create( CabinPos, CabinPos.Copy().Add( Vec2( 10, 10 ) ), 3 )
		Elevator.ElevatorCar2	= TLine.Create( CabinPos.Copy().Add( Vec2( 10, 10 ) ), CabinPos.Copy().Add( Vec2( 20, 0 ) ), 3 )
		Elevator.S1			= TSphere.Create( Vec2( Position.X + Width/2.0, Position.Y + Width/2.0 ), Width/2.0 )
		Elevator.S2			= TSphere.Create( Vec2( Position.X + Width/2.0, Position.Y + Height - Width/2.0 ), Width/2.0 )
		
		Elevator.ECDistance1  = Width/2.0
		Elevator.ECDistance2  = Sqr( ( Width/2.0 + 10 )^2 + 100 )
		Elevator.ECDistance3  = Width/2.0 + 20
		Elevator.ECDeltaAngle = -ATan2( -10, Width/2.0 + 10 )
		
		Elevator.Position = Position.Copy()
		Elevator.Size     = Vec2( Width, Height )
		
		Elevator.LastSecond     = CurrentTime()[ 6 .. ].ToInt()
		Elevator.LastSystemTime = MilliSecs()
		
		Elevator.Update()
		
		Physics.AddStaticBody( Elevator.ElevatorCar1 )
		Physics.AddStaticBody( Elevator.ElevatorCar2 )
		Physics.AddStaticBody( Elevator.S1 )
		Physics.AddStaticBody( Elevator.S2 )
		
		If Elevator.LastSecond >= 30 Then ..
			Physics.AddDynamicBody( TSphere.Create( Elevator.ElevatorCar1._X1.Copy().Add( Elevator.ElevatorCar2._X2 ).Multiply( 0.5 ), 10, 0.1 ) )
		
		Return Elevator
	End Function
End Type

Type TSphere Extends TDynamicBody
	Global SphereImg:TImage
	
	Field OldPosition:TVector
	Field Position:TVector
	Field Radius:Float
	Field Mass:Float
	
	Method New()
		Kind = KIND_SPHERE
	End Method
	
	Method Update()
		Local OldPos:TVector = Position.Copy()
		Position.Multiply( 2.0 ).Substract( OldPosition ).Add( Gravity )
		OldPosition = OldPos
	End Method
	
	Method ApplyImpulse( CollInfo:TCollisionInfo )
		Position.Substract( CollInfo.CollisionNormal.Multiply( CollInfo.CollisionLength ) )
	End Method
	
	Method GetMass:Float( Position:TVector )
		Return Mass
	End Method
	
	Method DetectIntersection:TCollisionInfo( DynamicBody:TDynamicBody )
		Select DynamicBody.Kind
			Case KIND_SEESAW
				Return SeesawIntersection( TSeesaw( DynamicBody ) )
			Case KIND_SPHERE
				Return SphereIntersection( TSphere( DynamicBody ) )
			Default
				Throw "Unknown body type!"
		End Select
	End Method
	
	Method SeesawIntersection:TCollisionInfo( Seesaw:TSeesaw )
		Local CollInfo:TCollisionInfo = Seesaw.SphereIntersection( Self )
		
		If CollInfo Then CollInfo.CollisionNormal.Multiply( -1 )
		
		Return CollInfo
	End Method
	
	Method SphereIntersection:TCollisionInfo( Sphere:TSphere )
		Local DistanceVector:TVector = Position.Copy().Substract( Sphere.Position )
		Local Distance:Float = DistanceVector.Length()
		
		If Distance < Radius + Sphere.Radius Then
			Return TCollisionInfo.Create( Null, DistanceVector.Normalize(), Radius + Sphere.Radius - Distance )
		Else
			Return Null
		EndIf
	End Method
	
	Method Render()
		SetScale Radius*2.0/SphereImg.Width, Radius*2.0/SphereImg.Height
		DrawImage SphereImg, Position.X - Radius, Position.Y - Radius
		
		SetScale 1, 1
	End Method
	
	Method GetBoundingBox( MinX:Float Var, MinY:Float Var, MaxX:Float Var, MaxY:Float Var )
		MinX = Position.X - Radius
		MinY = Position.Y - Radius
		MaxX = MinX + 2.0*Radius
		MaxY = MinY + 2.0*Radius
	End Method
	
	Function Create:TSphere( StartPosition:TVector, Radius:Float, Mass:Float = 1.0 )
		If Not SphereImg Then SphereImg = LoadImage( "Gfx/Sphere.png", FILTEREDIMAGE | MIPMAPPEDIMAGE )
		
		Local Sphere:TSphere = New TSphere
		
		Sphere.OldPosition = StartPosition.Copy()
		Sphere.Position    = StartPosition.Copy()
		Sphere.Radius      = Radius
		Sphere.Mass        = Mass
		
		Return Sphere
	End Function
End Type

Type TSeesaw Extends TDynamicBody
	Global ScrewImage:TImage
	
	Field _S1:TSphere
	Field _S2:TSphere
	Field _X1:TVector
	Field _X2:TVector
	Field _Hinge:Float
	Field _HingePos:TVector
	Field _Width:Float
	Field _Length:Float
	
	Field _Lambda:Float
	
	Field Image:TImage
	
	Method New()
		Kind = KIND_SEESAW
	End Method
	
	Method Update()
		_S1.Update()
		_S2.Update()
		
		CorrectAlignment()
	End Method
	
	Method ApplyImpulse( CollInfo:TCollisionInfo )
		Local MovementVector:TVector = CollInfo.CollisionNormal.Multiply( _Lambda*CollInfo.CollisionLength )
		Local Delta:TVector = _X2.Copy().Substract( _X1 )
		
		Local Interval:Float
		If Abs( Delta.X ) > Abs( Delta.Y ) Then
			Interval = ( CollInfo.Intersection.X - _X1.X )/Delta.X
		Else
			Interval = ( CollInfo.Intersection.Y - _X1.Y )/Delta.Y
		EndIf
		
		Local Ratio1:Float = _S2.Mass/( _S1.Mass + _S2.Mass )
		Local Ratio2:Float = _S1.Mass/( _S1.Mass + _S2.Mass )
		
		_X1.Substract( MovementVector.Copy().Multiply( ( 1 - Interval )*Ratio1 ) )
		_X2.Substract( MovementVector.Multiply( Interval*Ratio2 ) )
		
		CorrectAlignment()
	End Method
	
	Method CorrectAlignment()
		Local CurrentHingePos:TVector = _X2.Copy().Substract( _X1 ).Multiply( _Hinge ).Add( _X1 )
		Local MovementVector:TVector = _HingePos.Copy().Substract( CurrentHingePos ).Multiply( _Lambda )
		
		Local Ratio1:Float = _S1.Mass/( _S1.Mass + _S2.Mass )
		Local Ratio2:Float = _S2.Mass/( _S1.Mass + _S2.Mass )
		
		_X1.Add( MovementVector.Copy().Multiply( ( 1 -_Hinge )*Ratio2 ) )
		_X2.Add( MovementVector.Multiply( _Hinge*Ratio1 ) )
		
		Local Delta:TVector = _X2.Copy().Substract( _X1 )
		Local DeltaLength:Float = Delta.Length()
		
		DeltaLength = ( DeltaLength - _Length )/DeltaLength*0.5
		
		_X1.Add( Delta.Copy().Multiply( DeltaLength ) )
		_X2.Substract( Delta.Multiply( DeltaLength ) )
	End Method
	
	Method GetMass:Float( Position:TVector )
		Local Direction:TVector = _X2.Copy().Substract( _X1 ), Interval:Float
		
		If Abs( Direction.X ) > Abs( Direction.Y ) Then
			Interval = ( Position.X - _X1.X )/Direction.X
		Else
			Interval = ( Position.Y - _X1.Y )/Direction.Y
		EndIf
		
		If Interval >= 1.0 Then Return _S1.Mass
		If Interval <= 0.0 Then Return _S2.Mass
		
		Return _S1.Mass*Interval + _S2.Mass*( 1 - Interval )
	End Method
	
	Method DetectIntersection:TCollisionInfo( DynamicBody:TDynamicBody )
		Select DynamicBody.Kind
			Case KIND_SPHERE
				Return SphereIntersection( TSphere( DynamicBody ) )
			Case KIND_SEESAW
			Default
				Throw "Unkown body type!"
		End Select
	End Method
	
	Method SphereIntersection:TCollisionInfo( Sphere:TSphere )
		Local Distance:Float, CollVector:TVector, Intersection:TVector, Interval:Float
		
		Local Direction:TVector = _X2.Copy().Substract( _X1 )
		Local Normal:TVector = Vec2( -Direction.Y, Direction.X ).Normalize()
		
		If Abs( Direction.X ) > Abs( Direction.Y ) Then
			Interval = ( Sphere.Position.X - _X1.X )/Direction.X
		Else
			Interval = ( Sphere.Position.Y - _X1.Y )/Direction.Y
		EndIf
		
		If Interval >= 0.0 And Interval <= 1.0 Then
			Distance     = Sphere.Position.DotP( Normal ) - _X1.DotP( Normal )
			CollVector   = Normal.Copy()
			Intersection = Direction.Copy().Multiply( Interval ).Add( _X1 ).Add( CollVector.Copy().Multiply( Sgn( Distance )*_Width ) )
		Else
			Local TerminalPoint:TVector
			
			If Interval < 0.0 Then TerminalPoint = _X1 Else TerminalPoint = _X2
			
			Local PointSphere:TVector = Sphere.Position.Copy().Substract( TerminalPoint )
			
			Distance     = PointSphere.Length()
			CollVector   = PointSphere.Normalize()
			Intersection = TerminalPoint.Copy().Add( CollVector.Copy().Multiply( Distance - Sphere.Radius ) )
		EndIf
		
		If Abs( Distance ) < _Width + Sphere.Radius Then
			Local CollVectorLength:Float	= _Width + Sphere.Radius - Abs( Distance )
			If Distance > 0.0 Then CollVectorLength :* -1
			
			Return TCollisionInfo.Create( Intersection, CollVector, CollVectorLength )
		Else
			Return Null
		EndIf
	End Method
	
	Method SetWidth( Width:Float )
		_Width = Width
		
		If _S1 Then
			_S1.Radius = Width
			_S2.Radius = Width
		EndIf
	End Method
	
	Method SetTerminals( X1:TVector, X2:TVector, X1Mass:Float, X2Mass:Float )
		_S1 = TSphere.Create( X1, _Width, X1Mass )
		_S2 = TSphere.Create( X2, _Width, X2Mass )
		_X1 = _S1.Position
		_X2 = _S2.Position
		
		_Length = _X2.Copy().Substract( _X1 ).Length()
	End Method
	
	Method SetHinge( Hinge:Float )
		_Hinge = Hinge
		
		Local Ratio1:Float = _S1.Mass/( _S1.Mass + _S2.Mass )
		Local Ratio2:Float = _S2.Mass/( _S1.Mass + _S2.Mass )
		
		_HingePos = _X2.Copy().Substract( _X1 ).Multiply( _Hinge ).Add( _X1 )
		_Lambda   = 1.0/( Hinge*Hinge*Ratio1 + ( 1 - Hinge )*( 1 - Hinge )*Ratio2 )
		'_Lambda   = 1.0/( Hinge*Hinge*Ratio1*Ratio1 + ( 1 - Hinge )*( 1 - Hinge )*Ratio2*Ratio2 )
	End Method
	
	Method Render()
		Local Angle:Float = ATan2( _X2.Y - _X1.Y, _X2.X - _X1.X )
		
		SetRotation( Angle )
		SetImageHandle( Image, Image.Width*_Hinge, Image.Height/2.0 )
		DrawImage Image, _HingePos.X, _HingePos.Y
		
		SetRotation( 0 )
		
		DrawImage ScrewImage, Floor( _HingePos.X ) - 5, Floor( _HingePos.Y ) - 5
	End Method
	
	Method GetBoundingBox( MinX:Float Var, MinY:Float Var, MaxX:Float Var, MaxY:Float Var )
		MinX = Min( _X1.X, _X2.X ) - _Width
		MinY = Min( _X1.Y, _X2.Y ) - _Width
		MaxX = Max( _X1.X, _X2.X ) + _Width
		MaxY = Max( _X1.Y, _X2.Y ) + _Width
	End Method
	
	Function Create:TSeesaw( X1:TVector, X2:TVector, ImagePath:String, Width:Float = 1, Hinge:Float = 0.5, X1Mass:Float = 1.0, X2Mass:Float = 1.0 )
		If Not ScrewImage Then ScrewImage = LoadImage( "Gfx/Screw.png" )
		
		Local Seesaw:TSeesaw = New TSeesaw
		
		Seesaw.SetTerminals( X1, X2, X1Mass, X2Mass )
		Seesaw.SetWidth( Width )
		Seesaw.SetHinge( Hinge )
		Seesaw.Image = LoadImage( ImagePath, FILTEREDIMAGE )
		
		Return Seesaw
	End Function
End Type

Type TLine Extends TStaticBody
	Field _S1:TSphere
	Field _S2:TSphere
	Field _X1:TVector
	Field _X2:TVector
	Field _Width:Float
	
	Field Direction:TVector
	Field Normal:TVector
	
	Field C:Float
	
	Method DetectIntersection:TCollisionInfo( DynamicBody:TDynamicBody )
		Select DynamicBody.Kind
			Case KIND_SEESAW
				Return SeesawIntersection( TSeesaw( DynamicBody ) )
			Case KIND_SPHERE
				Return SphereIntersection( TSphere( DynamicBody ) )
			Default
				Throw "Unknown body type!"
		End Select
	End Method
	
	Method SeesawIntersection:TCollisionInfo( Seesaw:TSeesaw )
		Local CollInfo:TCollisionInfo = SphereIntersection( Seesaw._S1 )
		If CollInfo Then Return CollInfo
		
		CollInfo = SphereIntersection( Seesaw._S2 )
		If CollInfo Then Return CollInfo
		
		CollInfo = Seesaw.SphereIntersection( _S1 )
		If CollInfo Then CollInfo.CollisionNormal.Multiply( -1 ); Return CollInfo
		
		CollInfo = Seesaw.SphereIntersection( _S2 )
		If CollInfo Then CollInfo.CollisionNormal.Multiply( -1 ); Return CollInfo
	End Method
	
	Method SphereIntersection:TCollisionInfo( Sphere:TSphere )
		Local Distance:Float, CollVector:TVector, Intersection:TVector, Interval:Float
		
		If Abs( Direction.X ) > Abs( Direction.Y ) Then
			Interval = ( Sphere.Position.X - _X1.X )/Direction.X
		Else
			Interval = ( Sphere.Position.Y - _X1.Y )/Direction.Y
		EndIf
		
		If Interval >= 0.0 And Interval <= 1.0 Then
			Distance     = Sphere.Position.DotP( Normal ) + C
			CollVector   = Normal.Copy()
			Intersection = Direction.Copy().Multiply( Interval ).Add( _X1 ).Add( CollVector.Copy().Multiply( Sgn( Distance )*_Width ) )
		Else
			Local TerminalPoint:TVector
			
			If Interval < 0.0 Then TerminalPoint = _X1 Else TerminalPoint = _X2
			
			Local PointSphere:TVector = Sphere.Position.Copy().Substract( TerminalPoint )
			
			Distance     = PointSphere.Length()
			CollVector   = PointSphere.Normalize()
			Intersection = TerminalPoint.Copy().Add( CollVector.Copy().Multiply( Distance - Sphere.Radius ) )
		EndIf
		
		If Abs( Distance ) < _Width + Sphere.Radius Then
			Local CollVectorLength:Float	= _Width + Sphere.Radius - Abs( Distance )
			If Distance > 0.0 Then CollVectorLength :* -1
			
			Return TCollisionInfo.Create( Intersection, CollVector, CollVectorLength )
		Else
			Return Null
		EndIf
	End Method
	
	Method SetTerminals( X1:TVector, X2:TVector )
		_X1 = X1.Copy()
		_X2 = X2.Copy()
		_S1 = TSphere.Create( _X1, _Width )
		_S2 = TSphere.Create( _X2, _Width )
		
		Direction = _X2.Copy().Substract( _X1 )
		Normal = Vec2( -Direction.Y, Direction.X ).Normalize()
		
		C = -_X1.DotP( Normal )
	End Method
	
	Method SetWidth( Width:Float )
		_Width = Width
		
		_S1.Radius = Width
		_S2.Radius = Width
	End Method
	
	Method Render()
		SetLineWidth _Width*2.0
		DrawLine _X1.X, _X1.Y, _X2.X, _X2.Y
		DrawOval _X1.X - _Width, _X1.Y - _Width, _Width*2.0, _Width*2.0
		DrawOval _X2.X - _Width, _X2.Y - _Width, _Width*2.0, _Width*2.0
	End Method
	
	Method GetBoundingBox( MinX:Float Var, MinY:Float Var, MaxX:Float Var, MaxY:Float Var )
		MinX = Min( _X1.X, _X2.X ) - _Width
		MinY = Min( _X1.Y, _X2.Y ) - _Width
		MaxX = Max( _X1.X, _X2.X ) + _Width
		MaxY = Max( _X1.Y, _X2.Y ) + _Width
	End Method
	
	Function Create:TLine( X1:TVector, X2:TVector, Width:Float = 1 )
		Local Line:TLine = New TLine
		
		Line.SetTerminals( X1, X2 )
		Line.SetWidth( Width )
		
		Return Line
	End Function
End Type

Function InitClock:TSphereElevator( Physics:TPhysics ) 'Hardcoding vom feinsten *wrg*
	Physics.Clear()
	
	Physics.AddStaticBody( TLine.Create( Vec2( 23, 250 ), Vec2(  90, 360 ), 3.0 ) ) 'Rampe zwischen den beiden Liftrdern
	Physics.AddStaticBody( TLine.Create( Vec2( 70, 371 ), Vec2( 140, 371 ), 7.0 ) ) 'Verdickung unter der Rampe
	Physics.AddStaticBody( TLine.Create( Vec2( 90, 403 ), Vec2( 400, 393 ), 3.0 ) ) 'Lange Rampe ganz unten
	Physics.AddStaticBody( TLine.Create( Vec2( 67, 397 ), Vec2(  67, 400 ), 3.0 ) ) 'Stopper unter dem Lift
	Physics.AddStaticBody( TLine.Create( Vec2( -5, 385 ), Vec2(  10, 405 ), 6.0 ) ) 'Keil in der linken unteren Ecke
	
	Physics.AddStaticBody( TLine.Create( Vec2(  90, 363 ), Vec2( 367, 365 ), 3.0 ) ) 'Lange Rampe unter der untersten Wippe
	Physics.AddStaticBody( TLine.Create( Vec2( 410, 360 ), Vec2( 385, 395 ), 6.0 ) ) 'Keil in der rechten unteren Ecke
	
	Physics.AddDynamicBody( TSeesaw.Create( Vec2( 260, 124 ), Vec2( 390, 139 ), "Gfx/Seesaw1.png", 3, 0.80, 1.0, 5.2 ) ) 'Oberste Wippe, Minuten 0-6
	
	Physics.AddStaticBody( TLine.Create( Vec2( 100, 104.5 ), Vec2( 250, 123 ), 3 ) ) 'Rampe zur obersten Wippe
	Physics.AddStaticBody( TLine.Create( Vec2( 380, 110 ), Vec2( 380, 120 ), 3 ) ) 'Kugelstopper
	Physics.AddStaticBody( TLine.Create( Vec2( 390, 120 ), Vec2( 390, 125 ), 3 ) ) 'Drehbeschrnkung der Wippe nach unten
	Physics.AddStaticBody( TLine.Create( Vec2( 390, 145 ), Vec2( 400, 145 ), 3 ) ) 'Drehbeschrnkung der Wippe nach oben
	
	Physics.AddStaticBody( TLine.Create( Vec2( 170, 175 ), Vec2( 290, 167 ), 3 ) ) 'Rampe zur zweiten Wippe
	
	Physics.AddDynamicBody( TSeesaw.Create( Vec2( 200, 212 ), Vec2( 390, 230 ), "Gfx/Seesaw2.png", 3, 0.80, 1.0, 5.3 ) ) 'Mittlere Wippe, 6-Minuten-Einheiten
	
	Physics.AddStaticBody( TLine.Create( Vec2( 110, 197 ), Vec2( 190, 207 ), 3 ) ) 'Rampe zur Mittleren Wippe
	Physics.AddStaticBody( TLine.Create( Vec2( 390, 237 ), Vec2( 400, 237 ), 3 ) ) 'Drehbeschrnkung nach oben
	
	Physics.AddStaticBody( TLine.Create( Vec2( 160, 260 ), Vec2( 200, 256 ), 3 ) ) 'Rampe nach unten zur dritten Wippe
	Physics.AddStaticBody( TLine.Create( Vec2( 250, 246 ), Vec2( 250, 246 ), 3 ) ) 'Drehbeschrnkung nach unten
	
	Physics.AddDynamicBody( TSeesaw.Create( Vec2( 163, 291 ), Vec2( 390, 313 ), "Gfx/Seesaw3.png", 3, 0.80, 1.0, 5.5 ) ) 'Unterste Wippe, Stunden 0-12
	
	Physics.AddStaticBody( TLine.Create( Vec2( 110, 336 ), Vec2( 160, 335 ), 3 ) ) 'Rampe nach unten zum Kugeltank
	Physics.AddStaticBody( TLine.Create( Vec2( 260, 324 ), Vec2( 260, 324 ), 3 ) ) 'Drehbeschrnkung nach unten
	
	Physics.AddStaticBody( TLine.Create( Vec2( 110, 280 ), Vec2( 150, 285.5 ), 3 ) ) 'Rampe nach rechts zur Stundenwippe hin
	Physics.AddStaticBody( TLine.Create( Vec2( 390, 320 ), Vec2( 400,   320 ), 3 ) ) 'Drehbeschrnkung nach oben
	
	Physics.AddStaticBody( TLine.Create( Vec2(   0, -10 ), Vec2( 400, -10 ), 10 ) ) 'Obere Wand
	Physics.AddStaticBody( TLine.Create( Vec2(   0, 410 ), Vec2( 400, 410 ), 10 ) ) 'Untere Wand
	Physics.AddStaticBody( TLine.Create( Vec2( -12,   0 ), Vec2( -12, 410 ), 10 ) ) 'Linke Wand
	Physics.AddStaticBody( TLine.Create( Vec2( 410,   0 ), Vec2( 410, 400 ), 10 ) ) 'Rechte Wand
	
	Local Seesaw1:Int = CurrentTime()[ 3 .. 5 ].ToInt() Mod 6
	Local Seesaw2:Int = CurrentTime()[ 3 .. 5 ].ToInt()/6
	Local Seesaw3:Int = CurrentTime()[   .. 3 ].ToInt() Mod 12
	
	For Local I:Int = 0 To Seesaw1 - 1
		Physics.AddDynamicBody( TSphere.Create( Vec2( 365 - I*20, 122 - I*2 ), 10, 0.1 ) )
	Next
	
	For Local I:Int = 0 To Seesaw2 - 1
		Physics.AddDynamicBody( TSphere.Create( Vec2( 390 - I*20, 216 - I*2 ), 10, 0.1 ) )
	Next
	
	For Local I:Int = 0 To Seesaw3 - 1
		Physics.AddDynamicBody( TSphere.Create( Vec2( 390 - I*20, 300 - I*2 ), 10, 0.1 ) )
	Next
	
	Local RestCount:Int = 5 - Seesaw1 + 9 - Seesaw2 + 11 - Seesaw3 + 2
	
	For Local I:Int = 0 To Min( RestCount, 14 )
		Physics.AddDynamicBody( TSphere.Create( Vec2( 80 + I*21, 390 - I ), 10, 0.1 ) )
	Next
	
	For Local I:Int = 0 To RestCount - 14
		Physics.AddDynamicBody( TSphere.Create( Vec2( 373 - I*21, 350 ), 10, 0.1 ) )
	Next
	
	Local SphereElevator:TSphereElevator = TSphereElevator.Create( Physics, Vec2( 20, 50 ), 60, 330 )
	Physics.AddDynamicBody( SphereElevator ) 'Kugellift a.k.a. Sekundenzeiger
	
	Return SphereElevator
End Function